@@ -39,7 +39,7 @@
     STRIDE_C, INDEX_STRIDE, INDEX_BASE, BATCHSIZE) \
     ACC_BENCH_USEOMP(libxsmm_gemm_batch) \
     (IPREC, OPREC, TRANSA, TRANSB, M, N, K, ALPHA, A, LDA, STRIDE_A, B, LDB, STRIDE_B, BETA, C, LDC, STRIDE_C, INDEX_STRIDE, \
-      INDEX_BASE, BATCHSIZE, 0 /*batchcheck*/)
+      INDEX_BASE, BATCHSIZE)
 #  define PRINTF(...) \
     do { \
       const size_t print_buffer_size = sizeof(print_buffer) - print_offset; \
@@ -72,6 +72,9 @@
 #if !defined(NREPEAT)
 #  define NREPEAT 3
 #endif
+#if !defined(XREPEAT)
+#  define XREPEAT 66
+#endif
 #if !defined(TRANSPOSE)
 #  define TRANSPOSE
 #endif
@@ -86,10 +89,10 @@
 #endif
 
 #define ACC_BENCH_SMM_EPSILON(T) DBCSR_CONCATENATE(ACC_BENCH_SMM_EPSILON_, T)
-#define ACC_BENCH_SMM_EPSILON_double 3E-3
-#define ACC_BENCH_SMM_EPSILON_float 3E-3
+#define ACC_BENCH_SMM_EPSILON_double 1E-3
+#define ACC_BENCH_SMM_EPSILON_float 2E-3
 
-#define ROUNDUP2(N, NPOT) ((((unsigned long long)N) + ((NPOT)-1)) & ~((NPOT)-1))
+#define ROUNDUP2(N, NPOT) ((((unsigned long long)N) + ((NPOT) - 1)) & ~((NPOT) - 1))
 #define CHECK(EXPR, RPTR, VALUE) \
   do { \
     if (NULL != ((const void*)(RPTR))) { \
@@ -213,29 +216,31 @@ int main(int argc, char* argv[]) {
   const char *snc = NULL, *sna = NULL, *snb = NULL;
   FILE* file = NULL;
 #if defined(USE_LIBXSMM) && defined(VALIDATE)
-  double maxerror = 0;
+  double maxdiff = 0;
 #else
   DBCSR_MARK_USED(check);
 #endif
   CHECK(libsmm_acc_init(), &result, check); /* note: libsmm_acc_init() may imply acc_init() */
   if (EXIT_SUCCESS == result) {
-    const char* const env_device = getenv("DEVICE");
-    const int device = ((NULL == env_device || '\0' == *env_device) ? 0 : atoi(env_device));
     int ndevices = 0;
     result = c_dbcsr_acc_get_ndevices(&ndevices);
-    if (0 < ndevices && (0 == device || EXIT_SUCCESS == c_dbcsr_acc_set_active_device(device))) {
-#if defined(_DEBUG)
-      fprintf(stderr, "Activated device %i of %i (device%i).\n", device + 1, ndevices, device);
-#endif
-    }
-    else {
-      if (0 >= ndevices) {
-        fprintf(stderr, "ERROR: No ACC-device found!\n");
+    if (EXIT_SUCCESS == result && 0 < ndevices) {
+      const char* const env_device = getenv("DEVICE");
+      const char* const env_rank = (NULL != getenv("PMI_RANK") ? getenv("PMI_RANK") : getenv("OMPI_COMM_WORLD_LOCAL_RANK"));
+      const int rank = (NULL != env_rank ? atoi(env_rank) : -1);
+      int device = ((NULL == env_device || '\0' == *env_device) ? 0 : atoi(env_device));
+      device = ((0 <= device && device < ndevices) ? (0 <= rank ? (rank % ndevices) : device) : -1);
+      result = c_dbcsr_acc_set_active_device(device);
+      if (EXIT_SUCCESS == result) {
+        printf("Activated device%i (ndevices=%i)\n", device, ndevices);
       }
       else {
-        fprintf(stderr, "ERROR: Failed to activate device %i of %i!\n", device, ndevices);
+        fprintf(stderr, "ERROR: Failed to activate device!\n");
       }
-      result = EXIT_FAILURE;
+    }
+    else {
+      fprintf(stderr, "ERROR: No ACC-device found!\n");
+      if (EXIT_SUCCESS == result) result = EXIT_FAILURE;
     }
     if (EXIT_SUCCESS == result) {
       rnd = (int*)malloc(sizeof(int) * NRAND);
@@ -279,7 +284,7 @@ int main(int argc, char* argv[]) {
 #if defined(USE_LIBXSMM)
       libxsmm_timer_tickint start;
       int print_offset = 0;
-      char print_buffer[1024];
+      char print_buffer[1024] = "";
 #  if defined(__OPENCL)
       const char* const env_smm_repeat = getenv("SMM_NREPEAT");
       const int smm_nrepeat = (NULL == env_smm_repeat ? 1 : MAX(atoi(env_smm_repeat), 1));
@@ -292,7 +297,8 @@ int main(int argc, char* argv[]) {
       double duration = 0;
 #endif
       const char* const env_stack_size = getenv("SMM_BATCHSIZE");
-      int nrepeat = (0 < inr ? inr : NREPEAT);
+      const int xrepeat = (0 != check ? NREPEAT : XREPEAT);
+      int nrepeat = (0 < inr ? inr : xrepeat);
       int stack_size, na, nb, nc, nr, r;
       if (NULL == env_stack_size) {
         stack_size = 0;
@@ -325,7 +331,7 @@ int main(int argc, char* argv[]) {
           const int r = rnd[nok % NRAND], ss = -stack_size, bs = (1 < ss ? ss : BATCHSIZE);
           const int limit = (BATCHGRAIN < ss ? ((bs + BATCHGRAIN - 1) / BATCHGRAIN) : ss);
           stack_size = (r % limit + 1) * BATCHGRAIN;
-          nrepeat = MAX((BATCHSIZE * nrepeat + stack_size - 1) / stack_size, NREPEAT);
+          nrepeat = MAX((BATCHSIZE * nrepeat + stack_size - 1) / stack_size, xrepeat);
         }
         else stack_size = BATCHSIZE; /* plain default */
       }
@@ -480,18 +486,22 @@ int main(int argc, char* argv[]) {
             /* transfer result from device to host for validation */
             CHECK(c_dbcsr_acc_memcpy_d2h(cmat_dev, cmat_hst, sizeof(ELEM_TYPE) * mn * nc, stream), &result, check);
             CHECK(c_dbcsr_acc_stream_sync(stream), &result, check);
-#    if LIBXSMM_VERSION4(1, 17, 0, 0) < LIBXSMM_VERSION_NUMBER
+#    if defined(USE_LIBXSMM)
             if (EXIT_SUCCESS == result) {
               libxsmm_matdiff_info diff;
               /* validate result buffers at once (including excess/padded space) */
               result = libxsmm_matdiff(&diff, LIBXSMM_DATATYPE(ELEM_TYPE), mn, nc, gold_hst, cmat_hst, &mn, &mn);
               if (EXIT_SUCCESS == result) {
-                const double relerror = 1.0 - diff.rsq;
-                PRINTF("rel.error: %g", relerror);
-                if (maxerror < relerror && NULL != file) maxerror = relerror;
-                if (0 < relerror) {
+#      if defined(USE_LIBXSMM) && LIBXSMM_VERSION4(1, 17, 0, 0) < LIBXSMM_VERSION_NUMBER
+                const double epsilon = libxsmm_matdiff_epsilon(&diff); /* 1.0 - diff.rsq */
+#      else
+                const double epsilon = diff.normf_rel;
+#      endif
+                PRINTF("diff.cur: %g", epsilon);
+                if (maxdiff < epsilon && NULL != file) maxdiff = epsilon;
+                if (0 < epsilon) {
                   if (LIBXSMM_NOTNAN(diff.v_tst)) {
-                    PRINTF(" (%g != %g)\n", diff.v_ref, diff.v_tst);
+                    PRINTF(" (|%g-%g|=%g)\n", diff.v_ref, diff.v_tst, diff.linf_abs);
                   }
                   else {
                     PRINTF(" (%g)\n", diff.v_tst);
@@ -500,8 +510,9 @@ int main(int argc, char* argv[]) {
                 else {
                   PRINTF("\n");
                 }
-                if (0 < check && check < relerror) result = EXIT_FAILURE;
+                if (0 < check && check < epsilon) result = EXIT_FAILURE;
               }
+              else fprintf(stderr, "ERROR: failed to validate!\n");
             }
 #    endif
           }
@@ -546,7 +557,7 @@ int main(int argc, char* argv[]) {
 #endif
   CHECK(c_dbcsr_acc_finalize(), NULL, check);
 #if defined(USE_LIBXSMM) && defined(VALIDATE)
-  if (1 < nok) printf("\nmax.error: %g\n", maxerror);
+  if (1 < nok) printf("\ndiff.max: %g\n", maxdiff);
 #endif
   if (EXIT_SUCCESS != result) {
     if (NULL != file) fclose(file);
