@@ -1,19 +1,19 @@
-/*------------------------------------------------------------------------------------------------*
- * Copyright (C) by the DBCSR developers group - All rights reserved                              *
- * This file is part of the DBCSR library.                                                        *
- *                                                                                                *
- * For information on the license, see the LICENSE file.                                          *
- * For further information please visit https://dbcsr.cp2k.org                                    *
- * SPDX-License-Identifier: GPL-2.0+                                                              *
- *------------------------------------------------------------------------------------------------*/
+/*------------------------------------------------------------------------------------------------*/
+/* Copyright (C) by the DBCSR developers group - All rights reserved                              */
+/* This file is part of the DBCSR library.                                                        */
+/*                                                                                                */
+/* For information on the license, see the LICENSE file.                                          */
+/* For further information please visit https://dbcsr.cp2k.org                                    */
+/* SPDX-License-Identifier: GPL-2.0+                                                              */
+/*------------------------------------------------------------------------------------------------*/
 
-/*****************************************************************************
- *  Authors: Peter Messmer <pmessmer@nvidia.com>,                            *
- *           Nikolay Markovskiy <nmarkovskiy@nvidia.com>                     *
- *  Simplified, cleanup & refacture: Based on 'cusmm_dnt_small.h'            *
- *           Andreas Gloess <andreas.gloess@chem.uzh.ch>                     *
- *                                                                           *
- *****************************************************************************/
+/*****************************************************************************/
+/*  Authors: Peter Messmer <pmessmer@nvidia.com>,                            */
+/*           Nikolay Markovskiy <nmarkovskiy@nvidia.com>                     */
+/*  Simplified, cleanup & refacture: Based on 'cusmm_dnt_small.h'            */
+/*           Andreas Gloess <andreas.gloess@chem.uzh.ch>                     */
+/*                                                                           */
+/*****************************************************************************/
 
 #include "smm_acc_common.h"
 
@@ -51,12 +51,9 @@
  * - a_block and b_block are not copied directly from global memory to shared memory,
  *   but rather via registers
  */
-template < int m,  int n,  int k, int M, int N, int threads, int grouping, int minblocks>
-__global__ void
-__launch_bounds__(threads, minblocks)
-smm_acc_dnt_medium(const int* __restrict__ param_stack, int stack_size,
-     const double* __restrict__ a_data, const double* __restrict__ b_data, double* c_data){
-
+template<int m, int n, int k, int M, int N, int threads, int grouping, int minblocks>
+__global__ void __launch_bounds__(threads, minblocks) smm_acc_dnt_medium(const int* __restrict__ param_stack, int stack_size,
+  const double* __restrict__ a_data, const double* __restrict__ b_data, double* c_data) {
   /* Total number of elements in block matrices */
   const int mn = m * n; /* c_block */
   const int mk = m * k; /* a_block */
@@ -67,14 +64,14 @@ smm_acc_dnt_medium(const int* __restrict__ param_stack, int stack_size,
   const int tidx = threadIdx.x;
 
   /* Number of columns and rows used to divide c_block into tiles */
-  const int cmax = (n % N == 0)?  n / N: n / N + 1;
-  const int rmax = (m % M == 0)?  m / M: m / M + 1;
+  const int cmax = (n % N == 0) ? n / N : n / N + 1;
+  const int rmax = (m % M == 0) ? m / M : m / M + 1;
 
   /* buff_l and buff_r can overlap in the multiplication step
    * (still better than 'if' in inner loop, see ^ref1)
    */
-  const int buf_tmp = (mk + k * N * cmax < M * rmax * k + 1)? M * rmax * k + 1: mk + k * N * cmax;
-  const int buf_sz = (buf_tmp < mn)? mn: buf_tmp;
+  const int buf_tmp = (mk + k * N * cmax < M * rmax * k + 1) ? M * rmax * k + 1 : mk + k * N * cmax;
+  const int buf_sz = (buf_tmp < mn) ? mn : buf_tmp;
 
   /* Column and row starting index of the tile T this thread will compute
    * Each thread computes all elements of one tile T */
@@ -93,8 +90,8 @@ smm_acc_dnt_medium(const int* __restrict__ param_stack, int stack_size,
   const int knloads_remained = (kn - n_knloads * load_unroll_factor_2 * threads) / threads;
 
   /* ... */
-  const int mkloads_tail = ((mkloads_remained + n_mkloads * load_unroll_factor_1) * threads == mk)? 0: 1;
-  const int knloads_tail = ((knloads_remained + n_knloads * load_unroll_factor_2) * threads == kn)? 0: 1;
+  const int mkloads_tail = ((mkloads_remained + n_mkloads * load_unroll_factor_1) * threads == mk) ? 0 : 1;
+  const int knloads_tail = ((knloads_remained + n_knloads * load_unroll_factor_2) * threads == kn) ? 0 : 1;
 
   /* ... */
   const int m_loads_to_finish = n_mkloads * (load_unroll_factor_1 * threads) + mkloads_remained * threads;
@@ -103,7 +100,7 @@ smm_acc_dnt_medium(const int* __restrict__ param_stack, int stack_size,
   const int left_to_finish_2 = n_loads_to_finish + tidx;
 
   /* Number of parameters per stack entry in parameter stack */
-  const int  npar = 3;
+  const int npar = 3;
 
   /* If multiple warps (in HIP linguo, "wavefronts") are running a single block multiplication,
    * synchronization is needed */
@@ -124,7 +121,7 @@ smm_acc_dnt_medium(const int* __restrict__ param_stack, int stack_size,
   /* psp: parameter stack position:
    *      index in the parameter stack of the first parameter stack entry to be processed by this thread
    * nrun: number of runs: number of stack entries to process in this thread */
-  int  psp, nrun;
+  int psp, nrun;
   bool is_loaded = false;
 
   /* Arrays in shared memory
@@ -132,7 +129,7 @@ smm_acc_dnt_medium(const int* __restrict__ param_stack, int stack_size,
    * param_stack_s: shared memory buffer containing the stack entries this thread should process
    *                number of stack entries in param_stack_s = grouping,
    *                number of integers per stack entry: 3 */
-  __shared__ int    param_stack_s[npar * grouping];
+  __shared__ int param_stack_s[npar * grouping];
 
   /* buff: shared memory buffer containing a_block (block submatrix of A) and b_block to multiply */
   __shared__ double buff[buf_sz];
@@ -147,160 +144,161 @@ smm_acc_dnt_medium(const int* __restrict__ param_stack, int stack_size,
 
   /* Set the partial sum (tile T) to zero */
   /* WHY NO PRAGMA UNROLL ???? */
-  for (int i = 0; i < M * N; i++)
-    myc[i] = 0.0;
+  for (int i = 0; i < M * N; i++) myc[i] = 0.0;
 
   /* Load and pack stack data for current block from global memory into smem
    * Get parameter stack entries from index "psp" to "psp + (nrun-1)*npar + 2"
    * Each triplet indicates the beginning of a submatrix to multiply */
   psp = bidx * npar * grouping;
 #pragma unroll 3
-  for (int i = tidx; i < nrun; i += threads){
+  for (int i = tidx; i < nrun; i += threads) {
     // param_stack is 1-based, convert to 0-based here
-    param_stack_s[i * npar    ] = __ldg(&param_stack[psp + i * npar    ]) - 1; /* value = index in a_data */
+    param_stack_s[i * npar] = __ldg(&param_stack[psp + i * npar]) - 1; /* value = index in a_data */
     param_stack_s[i * npar + 1] = __ldg(&param_stack[psp + i * npar + 1]) - 1; /* value = index in b_data */
     param_stack_s[i * npar + 2] = __ldg(&param_stack[psp + i * npar + 2]) - 1; /* value = index in c_data */
   }
 
   /* Wait until all the data has been loaded */
-  if (need_sync) syncthreads ();
+  if (need_sync) syncthreads();
 
   /* In each run, we process one stack entry from param_stack_s */
-  for (int run = 0; run < nrun; run++){
-
+  for (int run = 0; run < nrun; run++) {
     /* Index in shared memory buffers to read from */
     psp = run * npar;
 
     int srcA, srcB;
 
-    if (!is_loaded){ /* If a_block, b_block are not loaded into registers yet */
+    if (!is_loaded) { /* If a_block, b_block are not loaded into registers yet */
 
       /* Index in a_data, b_data and c_data arrays
        * indicating where to fetch resp. write back matrix elements for this run
        * srcA, B, C corresponding to the starting indices (i.e. offsets) of block submatrices to multiply */
-      srcA = param_stack_s[psp    ];
+      srcA = param_stack_s[psp];
       srcB = param_stack_s[psp + 1];
 
-    /* Copy a_block, b_block from global memory to registers */
-    if (m == n){
+      /* Copy a_block, b_block from global memory to registers */
+      if (m == n) {
 #pragma unroll 3
-      for (int i = tidx; i < n_mkloads * (load_unroll_factor_1 * threads); i += load_unroll_factor_1 * threads){
+        for (int i = tidx; i < n_mkloads * (load_unroll_factor_1 * threads); i += load_unroll_factor_1 * threads) {
 #pragma unroll
-        for (int l = 0; l < load_unroll_factor_1; l++){
-          lba_r[l] = __ldg(&a_data[srcA + i + l * threads]);
-          lbb_r[l] = __ldg(&b_data[srcB + i + l * threads]);
+          for (int l = 0; l < load_unroll_factor_1; l++) {
+            lba_r[l] = __ldg(&a_data[srcA + i + l * threads]);
+            lbb_r[l] = __ldg(&b_data[srcB + i + l * threads]);
+          }
         }
-      }
 
 #pragma unroll 3
-      for (int i = n_mkloads * (load_unroll_factor_1 * threads) + tidx; i < m_loads_to_finish; i += mkloads_remained * threads){
+        for (int i = n_mkloads * (load_unroll_factor_1 * threads) + tidx; i < m_loads_to_finish; i += mkloads_remained * threads) {
 #pragma unroll
-        for (int l = 0; l < mkloads_remained; l++){
-          lba_r[l] = __ldg(&a_data[srcA + i + l * threads]);
-          lbb_r[l] = __ldg(&b_data[srcB + i + l * threads]);
+          for (int l = 0; l < mkloads_remained; l++) {
+            lba_r[l] = __ldg(&a_data[srcA + i + l * threads]);
+            lbb_r[l] = __ldg(&b_data[srcB + i + l * threads]);
+          }
         }
-      }
 
-      if (left_to_finish_1 < mk){
-        lba_r[load_unroll_factor_1 + mkloads_remained] = __ldg(&a_data[srcA + left_to_finish_1]);
-        lbb_r[load_unroll_factor_2 + knloads_remained] = __ldg(&b_data[srcB + left_to_finish_2]);
-      }
-    } else {
-#pragma unroll 3
-      for (int i = tidx; i < n_mkloads * (load_unroll_factor_1 * threads); i += load_unroll_factor_1 * threads){
-#pragma unroll
-        for (int l = 0; l < load_unroll_factor_1; l++){
-          lba_r[l] = __ldg(&a_data[srcA + i + l * threads]);
+        if (left_to_finish_1 < mk) {
+          lba_r[load_unroll_factor_1 + mkloads_remained] = __ldg(&a_data[srcA + left_to_finish_1]);
+          lbb_r[load_unroll_factor_2 + knloads_remained] = __ldg(&b_data[srcB + left_to_finish_2]);
         }
       }
+      else {
+#pragma unroll 3
+        for (int i = tidx; i < n_mkloads * (load_unroll_factor_1 * threads); i += load_unroll_factor_1 * threads) {
+#pragma unroll
+          for (int l = 0; l < load_unroll_factor_1; l++) {
+            lba_r[l] = __ldg(&a_data[srcA + i + l * threads]);
+          }
+        }
 
 #pragma unroll 3
-      for (int i = tidx; i < n_knloads * (load_unroll_factor_2 * threads); i += load_unroll_factor_2 * threads){
+        for (int i = tidx; i < n_knloads * (load_unroll_factor_2 * threads); i += load_unroll_factor_2 * threads) {
 #pragma unroll
-        for (int l = 0; l < load_unroll_factor_2; l++){
-          lbb_r[l] = __ldg(&b_data[srcB + i + l * threads]);
+          for (int l = 0; l < load_unroll_factor_2; l++) {
+            lbb_r[l] = __ldg(&b_data[srcB + i + l * threads]);
+          }
         }
-      }
 
 #pragma unroll 3
-      for (int i = n_mkloads * (load_unroll_factor_1 * threads) + tidx; i < m_loads_to_finish; i += mkloads_remained * threads){
+        for (int i = n_mkloads * (load_unroll_factor_1 * threads) + tidx; i < m_loads_to_finish; i += mkloads_remained * threads) {
 #pragma unroll
-        for (int l = 0; l < mkloads_remained; l++){
-          lba_r[l] = __ldg(&a_data[srcA + i + l * threads]);
+          for (int l = 0; l < mkloads_remained; l++) {
+            lba_r[l] = __ldg(&a_data[srcA + i + l * threads]);
+          }
         }
-      }
 
 #pragma unroll 3
-      for (int i = n_knloads * (load_unroll_factor_2 * threads) + tidx; i < n_loads_to_finish; i += knloads_remained * threads){
+        for (int i = n_knloads * (load_unroll_factor_2 * threads) + tidx; i < n_loads_to_finish; i += knloads_remained * threads) {
 #pragma unroll
-        for (int l = 0; l < knloads_remained; l++){
-          lbb_r[l] = __ldg(&b_data[srcB + i + l * threads]);
+          for (int l = 0; l < knloads_remained; l++) {
+            lbb_r[l] = __ldg(&b_data[srcB + i + l * threads]);
+          }
         }
-      }
 
-      if (left_to_finish_1 < mk) lba_r[load_unroll_factor_1 + mkloads_remained] = __ldg(&a_data[srcA + left_to_finish_1]);
-      if (left_to_finish_2 < kn) lbb_r[load_unroll_factor_2 + knloads_remained] = __ldg(&b_data[srcB + left_to_finish_2]);
-    }
+        if (left_to_finish_1 < mk) lba_r[load_unroll_factor_1 + mkloads_remained] = __ldg(&a_data[srcA + left_to_finish_1]);
+        if (left_to_finish_2 < kn) lbb_r[load_unroll_factor_2 + knloads_remained] = __ldg(&b_data[srcB + left_to_finish_2]);
+      }
 
       /* Wait until all the data has been loaded to registers */
-      syncthreads ();
-    } else {
-       is_loaded = false;
+      syncthreads();
+    }
+    else {
+      is_loaded = false;
     }
 
     /* Copy a_block, b_block from registers to shared memory */
-    if (m == n){
+    if (m == n) {
 #pragma unroll 3
-      for (int i = n_mkloads * (load_unroll_factor_1 * threads) + tidx; i < m_loads_to_finish; i += mkloads_remained * threads){
+      for (int i = n_mkloads * (load_unroll_factor_1 * threads) + tidx; i < m_loads_to_finish; i += mkloads_remained * threads) {
 #pragma unroll
-        for (int l = 0; l < mkloads_remained; l++){
+        for (int l = 0; l < mkloads_remained; l++) {
           buff[i + l * threads] = lba_r[l];
           buff[i + mk + l * threads] = lbb_r[l];
         }
       }
 
 #pragma unroll 3
-      for (int i = tidx; i < n_mkloads * (load_unroll_factor_1 * threads); i += load_unroll_factor_1 * threads){
+      for (int i = tidx; i < n_mkloads * (load_unroll_factor_1 * threads); i += load_unroll_factor_1 * threads) {
 #pragma unroll
-        for (int l = 0; l < load_unroll_factor_1; l++){
+        for (int l = 0; l < load_unroll_factor_1; l++) {
           buff[i + l * threads] = lba_r[l];
           buff[i + mk + l * threads] = lbb_r[l];
         }
       }
 
-      if (left_to_finish_1 < mk){
+      if (left_to_finish_1 < mk) {
         buff[left_to_finish_1] = lba_r[load_unroll_factor_1 + mkloads_remained];
         buff[mk + left_to_finish_2] = lbb_r[load_unroll_factor_2 + knloads_remained];
       }
-    } else {
+    }
+    else {
 #pragma unroll 3
-      for (int i = n_mkloads * (load_unroll_factor_1 * threads) + tidx; i < m_loads_to_finish; i += mkloads_remained * threads){
+      for (int i = n_mkloads * (load_unroll_factor_1 * threads) + tidx; i < m_loads_to_finish; i += mkloads_remained * threads) {
 #pragma unroll
-        for (int l = 0; l < mkloads_remained; l++){
+        for (int l = 0; l < mkloads_remained; l++) {
           buff[i + l * threads] = lba_r[l];
         }
       }
 
 #pragma unroll 3
-      for (int i = n_knloads * (load_unroll_factor_2 * threads) + tidx; i < n_loads_to_finish; i += knloads_remained * threads){
+      for (int i = n_knloads * (load_unroll_factor_2 * threads) + tidx; i < n_loads_to_finish; i += knloads_remained * threads) {
 #pragma unroll
-        for (int l = 0; l < knloads_remained; l++){
+        for (int l = 0; l < knloads_remained; l++) {
           buff[i + mk + l * threads] = lbb_r[l];
         }
       }
 
 #pragma unroll 3
-      for (int i = tidx; i < n_mkloads * (load_unroll_factor_1 * threads); i += load_unroll_factor_1 * threads){
+      for (int i = tidx; i < n_mkloads * (load_unroll_factor_1 * threads); i += load_unroll_factor_1 * threads) {
 #pragma unroll
-        for (int l = 0; l < load_unroll_factor_1; l++){
+        for (int l = 0; l < load_unroll_factor_1; l++) {
           buff[i + l * threads] = lba_r[l];
         }
       }
 
 #pragma unroll 3
-      for (int i = tidx; i < n_knloads * (load_unroll_factor_2 * threads); i += load_unroll_factor_2 * threads){
+      for (int i = tidx; i < n_knloads * (load_unroll_factor_2 * threads); i += load_unroll_factor_2 * threads) {
 #pragma unroll
-        for (int l = 0; l < load_unroll_factor_2; l++){
+        for (int l = 0; l < load_unroll_factor_2; l++) {
           buff[i + mk + l * threads] = lbb_r[l];
         }
       }
@@ -310,75 +308,76 @@ smm_acc_dnt_medium(const int* __restrict__ param_stack, int stack_size,
     }
 
     /* Wait until all the data has been loaded to shared memory */
-    if (need_sync) syncthreads ();
+    if (need_sync) syncthreads();
 
 
     int next_run = run + 1;
     if (next_run >= nrun) is_loaded = true;
 
-    if (!is_loaded || (run == 0 && nrun > 1)){
+    if (!is_loaded || (run == 0 && nrun > 1)) {
       /* Next parameter stack position */
       int next_psp = next_run * npar;
 
       /* Index in a_data, b_data and c_data arrays
        * indicating where to fetch resp. write back matrix elements for this run
        * srcA, B, C corresponding to the strting indices of block submatrices to multiply */
-      srcA = param_stack_s[next_psp    ];
+      srcA = param_stack_s[next_psp];
       srcB = param_stack_s[next_psp + 1];
 
       /* Buffering: copy the input data for the next run from global memory to registers */
-      if (m == n){
+      if (m == n) {
 #pragma unroll 3
-        for (int i = tidx; i < n_mkloads * (load_unroll_factor_1 * threads); i += load_unroll_factor_1 * threads){
+        for (int i = tidx; i < n_mkloads * (load_unroll_factor_1 * threads); i += load_unroll_factor_1 * threads) {
 #pragma unroll
-          for (int l = 0; l < load_unroll_factor_1; l++){
+          for (int l = 0; l < load_unroll_factor_1; l++) {
             lba_r[l] = __ldg(&a_data[srcA + i + l * threads]);
             lbb_r[l] = __ldg(&b_data[srcB + i + l * threads]);
           }
         }
 
 #pragma unroll 3
-        for (int i = n_mkloads * (load_unroll_factor_1 * threads) + tidx; i < m_loads_to_finish; i += mkloads_remained * threads){
+        for (int i = n_mkloads * (load_unroll_factor_1 * threads) + tidx; i < m_loads_to_finish; i += mkloads_remained * threads) {
 #pragma unroll
-          for (int l = 0; l < mkloads_remained; l++){
+          for (int l = 0; l < mkloads_remained; l++) {
             lba_r[l] = __ldg(&a_data[srcA + i + l * threads]);
-            lbb_r[l] = __ldg(&b_data[srcB + i +l * threads]);
+            lbb_r[l] = __ldg(&b_data[srcB + i + l * threads]);
           }
         }
 
-        if (left_to_finish_1 < mk){
+        if (left_to_finish_1 < mk) {
           lba_r[load_unroll_factor_1 + mkloads_remained] = __ldg(&a_data[srcA + left_to_finish_1]);
           lbb_r[load_unroll_factor_2 + knloads_remained] = __ldg(&b_data[srcB + left_to_finish_2]);
         }
-      } else {
+      }
+      else {
 #pragma unroll 3
-        for (int i = tidx; i < n_mkloads * (load_unroll_factor_1 * threads); i += load_unroll_factor_1 * threads){
+        for (int i = tidx; i < n_mkloads * (load_unroll_factor_1 * threads); i += load_unroll_factor_1 * threads) {
 #pragma unroll
-          for (int l = 0; l < load_unroll_factor_1; l++){
+          for (int l = 0; l < load_unroll_factor_1; l++) {
             lba_r[l] = __ldg(&a_data[srcA + i + l * threads]);
           }
         }
 
 #pragma unroll 3
-        for (int i = tidx; i < n_knloads * (load_unroll_factor_2 * threads); i += load_unroll_factor_2 * threads){
+        for (int i = tidx; i < n_knloads * (load_unroll_factor_2 * threads); i += load_unroll_factor_2 * threads) {
 #pragma unroll
-          for (int l = 0; l < load_unroll_factor_2; l++){
+          for (int l = 0; l < load_unroll_factor_2; l++) {
             lbb_r[l] = __ldg(&b_data[srcB + i + l * threads]);
           }
         }
 
 #pragma unroll 3
-        for (int i = n_mkloads * (load_unroll_factor_1 * threads) + tidx; i < m_loads_to_finish; i += mkloads_remained * threads){
+        for (int i = n_mkloads * (load_unroll_factor_1 * threads) + tidx; i < m_loads_to_finish; i += mkloads_remained * threads) {
 #pragma unroll
-          for (int l = 0; l < mkloads_remained; l++){
+          for (int l = 0; l < mkloads_remained; l++) {
             lba_r[l] = __ldg(&a_data[srcA + i + l * threads]);
           }
         }
 
 #pragma unroll 3
-        for (int i = n_knloads * (load_unroll_factor_2 * threads) + tidx; i < n_loads_to_finish; i += knloads_remained * threads){
+        for (int i = n_knloads * (load_unroll_factor_2 * threads) + tidx; i < n_loads_to_finish; i += knloads_remained * threads) {
 #pragma unroll
-          for (int l = 0; l < knloads_remained; l++){
+          for (int l = 0; l < knloads_remained; l++) {
             lbb_r[l] = __ldg(&b_data[srcB + i + l * threads]);
           }
         }
@@ -391,11 +390,11 @@ smm_acc_dnt_medium(const int* __restrict__ param_stack, int stack_size,
     }
 
     /* Do actual multiplication. */
-    if (c < cmax  && r < rmax){
-      for (int l = 0; l < k; l++){
+    if (c < cmax && r < rmax) {
+      for (int l = 0; l < k; l++) {
         /* Loop over all elements c_ij of tile T */
-        for (int i = 0; i < M; i++){
-          for (int j = 0; j < N; j++){
+        for (int i = 0; i < M; i++) {
+          for (int j = 0; j < N; j++) {
             /* Compute c_ij = sum_k (a_ik * b_kj) in shared memory */
             myc[N * i + j] += buff_l[l * m + M * r + i] * buff_r[l * n + N * c + j];
           }
@@ -404,18 +403,17 @@ smm_acc_dnt_medium(const int* __restrict__ param_stack, int stack_size,
     }
 
 
-    if (run == nrun - 1 || param_stack_s[psp + 2] != param_stack_s[psp + 2 + npar]){
-
+    if (run == nrun - 1 || param_stack_s[psp + 2] != param_stack_s[psp + 2 + npar]) {
       /* Index in c_data indicating where to write back matrix elements for this run */
       int srcC = param_stack_s[psp + 2];
 
-      if (M > 1 || N > 1){
+      if (M > 1 || N > 1) {
         if (need_sync) syncthreads();
         /* Decompress results to buffer and set tile elements back to 0 */
-        if (c < cmax  && r < rmax){
-          for (int i = 0; i < M; i++){
-            for (int j = 0; j < N; j++){
-              if (M * r + i < m && N * c + j < n){
+        if (c < cmax && r < rmax) {
+          for (int i = 0; i < M; i++) {
+            for (int j = 0; j < N; j++) {
+              if (M * r + i < m && N * c + j < n) {
                 buff[(N * c + j) * m + M * r + i] = myc[N * i + j];
                 myc[N * i + j] = 0.0;
               }
@@ -426,19 +424,19 @@ smm_acc_dnt_medium(const int* __restrict__ param_stack, int stack_size,
 
         /* Add results from shared memory buffer to global C block. */
 #pragma unroll
-        for (int i = tidx; i < mn; i += threads){
-          atomicAdd (&c_data[srcC + i], buff[i]);
+        for (int i = tidx; i < mn; i += threads) {
+          atomicAdd(&c_data[srcC + i], buff[i]);
         }
-      } else {
+      }
+      else {
         /* Add results from registers to global C block. */
 #pragma unroll
-        for (int i = tidx; i < mn; i += threads){
-          atomicAdd (&c_data[srcC + i], myc[0]);
+        for (int i = tidx; i < mn; i += threads) {
+          atomicAdd(&c_data[srcC + i], myc[0]);
         }
         myc[0] = 0.0;
       }
     }
-    if (need_sync) syncthreads ();
+    if (need_sync) syncthreads();
   }
-
 }
