@@ -1,5 +1,6 @@
 /*------------------------------------------------------------------------------------------------*/
 /* Copyright (C) by the DBCSR developers group - All rights reserved                              */
+/* Copyright (C) 2022 Advanced Micro Devices, Inc. - All rights reserved                          */
 /* This file is part of the DBCSR library.                                                        */
 /*                                                                                                */
 /* For information on the license, see the LICENSE file.                                          */
@@ -51,7 +52,7 @@ inline int launch_kernel_from_handle(
 }
 
 //===========================================================================
-inline void validate_kernel(ACC_DRV(function) & kern_func, ACC_DRV(stream) stream, int threads, int grouping, int m, int n, int k) {
+inline int validate_kernel(ACC_DRV(function) & kern_func, ACC_DRV(stream) stream, int threads, int grouping, int m, int n, int k) {
   libsmm_acc_benchmark_t* h;
   libsmm_acc_benchmark_init(&h, test, m, n, k);
 
@@ -78,10 +79,11 @@ inline void validate_kernel(ACC_DRV(function) & kern_func, ACC_DRV(stream) strea
   double sumGPU = checkSum(h->mat_c, h->n_c, m, n);
   libsmm_acc_benchmark_finalize(h);
   if (sumGPU != sumCPU) {
-    printf("Kernel validation failed for multiplication kernel %ix%ix%i\nchecksum CPU: %g, checksum GPU: %g\nchecksum_diff: %g\n",
-      m, n, k, sumCPU, sumGPU, sumGPU - sumCPU);
-    exit(1);
+    //    printf("Kernel validation failed for multiplication kernel %ix%ix%i\nchecksum CPU: %g, checksum GPU: %g\nchecksum_diff: %g\n",
+    //      m, n, k, sumCPU, sumGPU, sumGPU - sumCPU);
+    return 1;
   }
+  return 0;
 }
 
 //===========================================================================
@@ -140,8 +142,8 @@ inline void jit_kernel(ACC_DRV(function) & kern_func, libsmm_acc_algo algo, int
   const char* compileOptions[] = {"-D__CUDA", "-w", ARCH_OPTION};
   size_t nOptions = 3;
 #else
-  const char* compileOptions[] = {"-D__HIP"};
-  size_t nOptions = 1;
+  const char* compileOptions[] = {"-D__HIP", "-O3", "-w", "-munsafe-fp-atomics"};
+  size_t nOptions = 4;
 #endif
   ACC_RTC(Result) compileResult = ACC_RTC(CompileProgram)(kernel_program, nOptions, compileOptions);
   if (compileResult != ACC_RTC_SUCCESS) {
@@ -194,30 +196,58 @@ inline void jit_kernel(ACC_DRV(function) & kern_func, libsmm_acc_algo algo, int
 
 
 kernel_map_iterator add_kernel_handle_to_jitted_kernels(
-  ACC_DRV(function) kern_func, ACC_DRV(stream) stream, Triplet h_mnk, int& threads, int& grouping) {
+  ACC_DRV(function) kern_func, ACC_DRV(stream) stream, Triplet h_mnk, int& threads, int& grouping, bool& generated_acc_untuned) {
   kernel_map_iterator kernel_it = kernel_handles.end();
 
+  // Check if the kernel was already generated and failed or if it is too big
+  if (failed_acc_kernels.find(h_mnk) != failed_acc_kernels.end()) return kernel_it;
+
+  libsmm_acc_algo algo;
+  int tile_m, tile_n, w, v, minblocks;
+
   // Check whether autotuned parameters are given for this kernel, and if so, retrieve them
   if (ht.find(h_mnk) != ht.end()) {
     // Retrieve launching parameters
     const KernelParameters params = ht.at(h_mnk);
-    libsmm_acc_algo algo = libsmm_acc_algo(params[0]); // enum {largeDB1, largeDB2, medium, small, tiny}
-    int tile_m = params[1];
-    int tile_n = params[2];
-    int w = params[3];
-    int v = params[4];
+    algo = libsmm_acc_algo(params[0]); // enum {largeDB1, largeDB2, medium, small, tiny}
+    tile_m = params[1];
+    tile_n = params[2];
+    w = params[3];
+    v = params[4];
     threads = params[5];
     grouping = params[6];
-    int minblocks = params[7];
-
-    // JIT and validate the kernel
-    jit_kernel(kern_func, algo, tile_m, tile_n, w, v, threads, grouping, minblocks, h_mnk[0], h_mnk[1], h_mnk[2]);
-    validate_kernel(kern_func, stream, threads, grouping, h_mnk[0], h_mnk[1], h_mnk[2]);
+    minblocks = params[7];
+    generated_acc_untuned = false;
+  }
+  else if (h_mnk[0] < 50 && h_mnk[1] < 50 && h_mnk[2] < 50) { // Use a default untuned kernel
+    algo = medium;
+    tile_m = 2;
+    tile_n = 2;
+    w = 0;
+    v = 0;
+    threads = 256;
+    grouping = 30;
+    minblocks = 1;
+    generated_acc_untuned = true;
+  }
+  else {
+    failed_acc_kernels.insert(h_mnk);
+    return kernel_it;
+  }
 
+  // JIT and validate the kernel
+  jit_kernel(kern_func, algo, tile_m, tile_n, w, v, threads, grouping, minblocks, h_mnk[0], h_mnk[1], h_mnk[2]);
+  if (validate_kernel(kern_func, stream, threads, grouping, h_mnk[0], h_mnk[1], h_mnk[2]) == 0) {
     // Store the handle to the JIT-ed kernel
     auto kernel_it_emplaced = kernel_handles.emplace(h_mnk, kernel_launcher(kern_func, threads, grouping));
     kernel_it = kernel_it_emplaced.first;
   }
+  else {
+    // The generated kernel gave wrong values, discard it
+    free(kern_func);
+    generated_acc_untuned = false;
+    failed_acc_kernels.insert(h_mnk);
+  }
 
   return kernel_it;
 }
@@ -255,6 +285,8 @@ int libsmm_acc_process_d(const int* param_stack, int stack_size, ACC_DRV(stream)
   Triplet h_mnk = {m, n, k};
   kernel_map_iterator kernel_it;
 
+  bool generated_acc_untuned = false;
+
 #if defined _OPENMP
 #  pragma omp critical(jit_multiplication)
 #endif
@@ -263,7 +295,7 @@ int libsmm_acc_process_d(const int* param_stack, int stack_size, ACC_DRV(stream)
     kernel_it = kernel_handles.find(h_mnk);
     if (kernel_it == kernel_handles.end()) { // the kernel has not been JIT-ed yet
 
-      kernel_it = add_kernel_handle_to_jitted_kernels(kern_func, stream, h_mnk, threads, grouping);
+      kernel_it = add_kernel_handle_to_jitted_kernels(kern_func, stream, h_mnk, threads, grouping, generated_acc_untuned);
 
     } // if the kernel could be jited successfully, the kernel_it iterator now points to the kernel_launcher.
     // if this wasn't possible, is set to kernel_handles.end()
@@ -282,7 +314,9 @@ int libsmm_acc_process_d(const int* param_stack, int stack_size, ACC_DRV(stream)
     // Construct argument pointer list and launch kernel
     void* args[] = {&param_stack, &stack_size, &a_data, &b_data, &c_data};
 
-    return launch_kernel_from_handle(kern_func, ((stack_size + grouping - 1) / grouping), threads, stream, args);
+    int return_value = launch_kernel_from_handle(kern_func, ((stack_size + grouping - 1) / grouping), threads, stream, args);
+
+    return ((return_value != 0) or (!generated_acc_untuned)) ? return_value : 10; // return 10 for generated default untuned kernel
   }
 }
 
@@ -296,9 +330,10 @@ int libsmm_acc_process(const int* param_stack_host, const int* param_stack_dev,
       // maximum size over any dimension
       return (libsmm_acc_process_blas((const int*)param_stack_host, stack_size, *((ACC_DRV(stream)*)c_stream), m, n, k,
         max_kernel_dim, (const double*)a_data, (const double*)b_data, (double*)c_data));
-    else
+    else {
       return (libsmm_acc_process_d((const int*)param_stack_dev, stack_size, *((ACC_DRV(stream)*)stack_stream), m, n, k,
         (const double*)a_data, (const double*)b_data, (double*)c_data));
+    }
   }
   return -10; // datatype not supported
 }
