@@ -5,13 +5,16 @@
 #                                                                                                  #
 # For information on the license, see the LICENSE file.                                            #
 # For further information please visit https://dbcsr.cp2k.org                                      #
-# SPDX-License-Identifier: GPL-2.0+                                                                #
+# SPDX-License-Identifier: BSD-3-Clause                                                            #
 ####################################################################################################
 # shellcheck disable=SC2048,SC2129
 
 BASENAME=$(command -v basename)
+DIRNAME=$(command -v dirname)
+HEAD=$(command -v head)
 SORT=$(command -v sort)
 SED=$(command -v gsed)
+CAT=$(command -v cat)
 CPP=$(command -v cpp)
 TR=$(command -v tr)
 RM=$(command -v rm)
@@ -28,17 +31,78 @@ if [ ! "${SED}" ]; then
   SED=$(command -v sed)
 fi
 
-if [ "${BASENAME}" ] && [ "${SORT}" ] && [ "${SED}" ] && \
-   [ "${TR}" ] && [ "${RM}" ] && [ "${WC}" ];
+trap_exit() {
+  if [ "0" != "$?" ] && [ "${HFILE}" ]; then ${RM} -f "${OFILE}"; fi
+}
+
+process_pre() {
+  if [ "$1" ]; then
+    if [ "${CPP}" ] && \
+       [ "$(eval "${CPP} ${CPPBASEFLAGS} $1" 2>/dev/null >/dev/null && echo "YES")" ];
+    then
+      if [ "${CPPFLAGS}" ] && \
+         [ "$(eval "${CPP} ${CPPFLAGS} ${CPPBASEFLAGS} $1" 2>/dev/null >/dev/null && echo "YES")" ];
+      then
+        eval "${CPP} ${CPPFLAGS} ${CPPBASEFLAGS} $1" 2>/dev/null
+      else
+        eval "${CPP} ${CPPBASEFLAGS} $1" 2>/dev/null
+      fi
+    else # fallback to sed
+      ${SED} -r ':a;s%(.*)/\*.*\*/%\1%;ta;/\/\*/!b;N;ba' "$1"
+    fi | \
+    if [ ! "$2" ] || [ "0" = "$2" ]; then # strip include guards
+      GUARD=$(${BASENAME} "$1" | ${TR} '[:lower:]' '[:upper:]' | ${TR} '.' '_')
+      if [ "${GUARD}" ] && [ "$(${SED} -n "/${GUARD}/p" "$1")" ]; then
+        ${SED} "/${GUARD}/d;\${/\s*\#\s*endif/d}"
+      else
+        ${CAT}
+      fi
+    else
+      ${CAT}
+    fi
+  fi
+}
+
+process() {
+  IFS=$'\n'
+  while read -r LINE; do
+    INCLUDE=$(${SED} -n "s/#[[:space:]]*include[[:space:]][[:space:]]*\"/\"/p" <<<"${LINE}")
+    if [ "${INCLUDE}" ] && [ "$1" ] && [ -e "$1" ]; then
+      CLINC=$(${SED} "s/\"//g" <<<"${INCLUDE}")
+      CLPATH=$(${DIRNAME} "$1")
+      FILE=${CLPATH}/${CLINC}
+      if [ "${FILE}" ] && [ -e "${FILE}" ]; then
+        process_pre "${FILE}" "$2" | process "${FILE}" "$2"
+      else
+        >&2 echo "WARNING: header file ${FILE} not found!"
+        #exit 1
+      fi
+    else
+      ${SED} <<<"${LINE}" \
+        -e '/^[[:space:]]*$/d' -e 's/[[:space:]]*$//' \
+        -e 's/[[:space:]]*\\/ \\\\/g' -e 's/"/\\"/g' \
+        -e 's/^/  "/' -e 's/$/\\n" \\/'
+    fi
+  done
+  unset IFS
+}
+
+if [ "${BASENAME}" ] && [ "${DIRNAME}" ] && [ "${HEAD}" ] && [ "${SORT}" ] && \
+   [ "${SED}" ] && [ "${CAT}" ] && [ "${TR}" ] && [ "${RM}" ] && [ "${WC}" ];
 then
   for OFILE in "$@"; do :; done
   while test $# -gt 0; do
     case "$1" in
     -h|--help)
       shift $#;;
+    -k|--keep)
+      KEEP=1
+      shift;;
+    -b|--banner)
+      BANNER=$2
+      shift 2;;
     -p|--params)
-      PARAMPATH=yes
-      PARAMS=$2
+      PARAMS="$2\t"
       shift 2;;
     -c|-d|--debug|--comments)
       CPPFLAGS+=" -C"
@@ -49,14 +113,10 @@ then
     *) break;;
     esac
   done
-  HERE="$(cd "$(dirname "$0")" && pwd -P)"
-  PARAMDIR=$(if [ ! "${PARAMDIR}" ]; then echo "${HERE}/smm/params"; fi)
-  if [ "${PARAMPATH}" ]; then
-    PARAMPATH=${PARAMS}
-  else
-    HERE="$(cd "$(dirname "$0")" && pwd -P)"
-    PARAMPATH=${PARAMDIR}
-  fi
+  HERE="$(cd "$(${DIRNAME} "$0")" && pwd -P)"
+  PARAMDIR=${PARAMDIR:-${PARAMS}}
+  PARAMDIR=${PARAMDIR:-${HERE}/smm/params}
+  PARAMDIR=$(echo -e "${PARAMDIR}" | ${TR} -d '\t')
   if [ "$#" -gt 1 ]; then
     # allow for instance /dev/stdout
     if [ "${OFILE##*.}" = "h" ]; then
@@ -75,45 +135,40 @@ then
         echo "$0 $*"
       fi
     fi
+    trap 'trap_exit' EXIT
+    RNAME=$(${BASENAME} "$(cd "$(${DIRNAME} "$1")" && pwd -P)")
+    ANAME=$(${TR} '[:lower:]' '[:upper:]' <<<"${RNAME}")
     NFILES_OCL=0
     for CLFILE in ${*:1:${#@}-1}; do
       if [ "${CLFILE##*.}" = "cl" ]; then
+        CLEXT=".cl"
+      elif [ "${CLFILE##*.}" = "h" ]; then
+        CLEXT=".h"
+      else
+        CLEXT=""
+      fi
+      if [ "${CLEXT}" ]; then
         if [ -e "${CLFILE}" ]; then
-          BNAME=$(${BASENAME} "${CLFILE}" .cl)
-          UNAME=$(echo "${BNAME}" | ${TR} '[:lower:]' '[:upper:]')
-          SNAME=OPENCL_LIBSMM_STRING_${UNAME}
-          VNAME=opencl_libsmm_source_${BNAME}
-          MNAME=OPENCL_LIBSMM_SOURCE_${UNAME}
+          CNAME=$(${BASENAME} "${CLFILE}" "${CLEXT}" | ${SED} "s/${RNAME}_//;s/_opencl//")
+          BNAME=$(${TR} '[:lower:]' '[:upper:]' <<<"${CNAME}")
+          SNAME=OPENCL_${ANAME}_STRING_${BNAME}
+          VNAME=opencl_${RNAME}_source_${CNAME}
+          MNAME=OPENCL_${ANAME}_SOURCE_${BNAME}
           if [ "0" != "$((0<(NFILES_OCL)))" ]; then
-            echo >>"${OFILE}"
+            echo
+          elif [ "${BANNER}" ] && [ "0" != "${BANNER}" ]; then
+            ${HEAD} -n"${BANNER}" "${CLFILE}"
           fi
-          echo "#define ${MNAME} ${VNAME}" >>"${OFILE}"
-          echo "#define ${SNAME} \\" >>"${OFILE}"
-          if [ "${CPP}" ] && \
-             [ "$(eval "${CPP} ${CPPBASEFLAGS} ${CLFILE}" 2>/dev/null >/dev/null && echo "YES")" ];
-          then
-            if [ "" != "${CPPFLAGS}" ] && \
-               [ "$(eval "${CPP} ${CPPFLAGS} ${CPPBASEFLAGS} ${CLFILE}" 2>/dev/null >/dev/null && echo "YES")" ];
-            then
-              eval "${CPP} ${CPPFLAGS} ${CPPBASEFLAGS} ${CLFILE}" 2>/dev/null
-            else
-              eval "${CPP} ${CPPBASEFLAGS} ${CLFILE}" 2>/dev/null
-            fi
-          else # fallback to sed
-            ${SED} -r ':a;s%(.*)/\*.*\*/%\1%;ta;/\/\*/!b;N;ba' "${CLFILE}"
-          fi | \
-          ${SED} \
-            -e '/^[[:space:]]*$/d' -e 's/[[:space:]]*$//' \
-            -e 's/\\/\\\\/g' -e 's/"/\\"/g' -e 's/^/  "/' -e 's/$/\\n" \\/' \
-            >>"${OFILE}"
-          echo "  \"\"" >>"${OFILE}"
-          echo "static const char ${VNAME}[] = ${SNAME};" >>"${OFILE}"
+          echo "#define ${MNAME} ${VNAME}"
+          echo "#define ${SNAME} \\"
+          process_pre "${CLFILE}" "${KEEP}" | process "${CLFILE}" "${KEEP}"
+          echo "  \"\""
+          echo "static const char ${VNAME}[] = ${SNAME};"
           NFILES_OCL=$((NFILES_OCL+1))
         else
           >&2 echo "ERROR: ${CLFILE} does not exist!"
-          if [ "${HFILE}" ]; then ${RM} -f "${OFILE}"; fi
           exit 1
-        fi
+        fi >>"${OFILE}"
       else
         CSVFILES=("${*:NFILES_OCL+1:${#@}-NFILES_OCL-1}")
         break
@@ -121,30 +176,28 @@ then
     done
     if [ "0" = "${NFILES_OCL}" ]; then
       >&2 echo "ERROR: no OpenCL file was given!"
-      if [ "${HFILE}" ]; then ${RM} -f "${OFILE}"; fi
       exit 1
     fi
     NFILES_CSV=0
     for CSVFILE in "${CSVFILES[@]}"; do
       if [ "${CSVFILE##*.}" = "csv" ]; then
-        if [ -e "${CSVFILE}" ]; then
+        if [ -f "${CSVFILE}" ]; then
           NFILES_CSV=$((NFILES_CSV+1))
         fi
       else
         >&2 echo "ERROR: ${CSVFILE} is not a CSV file!"
-        if [ "${HFILE}" ]; then ${RM} -f "${OFILE}"; fi
         exit 1
       fi
     done
-    if [ "0" = "${NFILES_CSV}" ] && [ "${PARAMPATH}" ]; then
-      CSVFILES=("${PARAMPATH}"/*.csv)
+    if [ "0" = "${NFILES_CSV}" ] && [ "${PARAMDIR}" ] && [ -d "${PARAMDIR}" ]; then
+      CSVFILES=("${PARAMDIR}"/*.csv)
       NFILES_CSV=${#CSVFILES[@]}
     fi
     for CSVFILE in "${CSVFILES[@]}"; do
       if [ ! "${DELIM}" ]; then
-        SEPAR=$(${SED} -n "1s/[^${DELIMS}]//gp" "${CSVFILE}")
+        SEPAR=$(${SED} -n "1s/[^${DELIMS}]//gp" "${CSVFILE}" 2>/dev/null)
         DELIM=${SEPAR:0:1}
-        MATCH=$(${SED} -n "1s/[^${DELIM}]//gp" "${CSVFILE}")
+        MATCH=$(${SED} -n "1s/[^${DELIM}]//gp" "${CSVFILE}" 2>/dev/null)
       fi
       if [ "${DELIM}" ]; then
         CHECK=$(${SED} "/^[[:space:]]*$/d;s/[^${DELIM}]//g" "${CSVFILE}" | ${SORT} -u | ${SED} -n "0,/./p")
@@ -156,49 +209,50 @@ then
       else
         ERRFILE=${CSVFILE}
       fi
-      if [ "${ERRFILE}" ]; then
+      if [ "${ERRFILE}" ] && [ -f "${ERRFILE}" ]; then
         >&2 echo "ERROR: ${ERRFILE} is malformed!"
-        if [ "${HFILE}" ]; then ${RM} -f "${OFILE}"; fi
         exit 1
       fi
     done
     DEVPAT="s/${DELIM}..*//"
     DEVICES=$(for CSVFILE in "${CSVFILES[@]}"; do ${SED} "1d;/^[[:space:]]*$/d;${DEVPAT}" "${CSVFILE}"; done | ${SORT} -u)
-    SNAME=OPENCL_LIBSMM_STRING_PARAMS_SMM
-    VNAME=opencl_libsmm_params_smm
-    DNAME=opencl_libsmm_devices
-    MNAME=$(echo "${VNAME}" | ${TR} '[:lower:]' '[:upper:]')
-    NNAME=$(echo "${DNAME}" | ${TR} '[:lower:]' '[:upper:]')
+    SNAME=OPENCL_${ANAME}_STRING_PARAMS_SMM
+    VNAME=opencl_${RNAME}_params_smm
+    DNAME=opencl_${RNAME}_devices
+    MNAME=$(${TR} '[:lower:]' '[:upper:]' <<<"${VNAME}")
+    NNAME=$(${TR} '[:lower:]' '[:upper:]' <<<"${DNAME}")
     if [ "${DEVICES}" ]; then
-      echo >>"${OFILE}"
-      echo "#define ${MNAME} ${VNAME}" >>"${OFILE}"
-      echo "#define ${SNAME} \\" >>"${OFILE}"
+      echo
+      echo "#define ${MNAME} ${VNAME}"
+      echo "#define ${SNAME} \\"
       CSVLINES=$(for CSVFILE in "${CSVFILES[@]}"; do ${SED} "1d;/^[[:space:]]*$/d;s/[\r]*$/\\\n\" \\\/" "${CSVFILE}"; done)
       IFS=$'\n'
       for LINE in ${CSVLINES}; do
-        I=0; IDEVICE=$(echo "${LINE}" | ${SED} "${DEVPAT}")
+        I=0; IDEVICE=$(${SED} "${DEVPAT}" <<<"${LINE}")
         for DEVICE in ${DEVICES}; do
           if [ "${DEVICE}" = "${IDEVICE}" ]; then break; fi
           I=$((I+1));
         done
-        echo "${LINE}" | ${SED} "s/[^${DELIM}]*//;s/^/  \"${I}/" >>"${OFILE}"
+        ${SED} "s/[^${DELIM}]*/  \"${I}/" <<<"${LINE}"
       done
-      echo "  \"\"" >>"${OFILE}"
-      echo "static const char ${VNAME}[] = ${SNAME};" >>"${OFILE}"
-      echo >>"${OFILE}"
-      echo "#define ${NNAME} ${DNAME}" >>"${OFILE}"
-      echo "static const char *const ${DNAME}[] = {" >>"${OFILE}"
-      I=0; S=","; NDEVICES=$(echo "${DEVICES}" | ${WC} -l)
+      echo "  \"\""
+      echo "static const char ${VNAME}[] = ${SNAME};"
+      echo
+      echo "#define ${NNAME} ${DNAME}"
+      echo "static const char *const ${DNAME}[] = {"
+      I=0; S=","; NDEVICES=$(${WC} -l <<<"${DEVICES}")
       for DEVICE in ${DEVICES}; do
         I=$((I+1)); if [ "0" != "$((NDEVICES==I))" ]; then S=""; fi
-        echo "  \"${DEVICE}\"${S}" >>"${OFILE}"
+        echo "  \"${DEVICE}\"${S}"
       done
       unset IFS
-      echo "};" >>"${OFILE}"
-    fi
+      echo "};"
+    fi >>"${OFILE}"
   else
     echo "Usage: $0 infile.cl [infile2.cl .. infileN.cl] [infile.csv [.. infileN.csv]] outfile.h"
     echo "       At least one OpenCL file and one header file must be supplied."
+    echo "       -k|--keep: do not strip include guards (stripped even if necessary)"
+    echo "       -b|--banner N: number of lines used as banner (default: 0)"
     echo "       -p|--params P: directory-path to CSV-files (can be \"\")"
     echo "             default: ${PARAMDIR}"
     echo "       -c|-d|--debug|--comments: keep comments in source-code"
